
#ifndef COLE_VISHKIN_H
#define COLE_VISHKIN_H

#include "cole_vishkin_message.h"

template<
   typename OsModel_P,
   typename Radio_P,
   typename Neighborhood_P,
   typename Debug_P,
   typename LabelDebug_P
>
class ColeVishkin {
   public:
      typedef OsModel_P OsModel;
      typedef Radio_P Radio;
      typedef Neighborhood_P Neighborhood;
      typedef Debug_P Debug;
      typedef LabelDebug_P LabelDebug;
      
      typedef ColeVishkin<OsModel, Radio, Neighborhood, Debug, LabelDebug> self_type;
      typedef self_type* self_pointer_t;
      
      typedef typename Radio::node_id_t color_t;
      typedef typename Radio::node_id_t node_id_t;
      typedef typename Radio::size_t size_t;
      typedef typename Radio::block_data_t block_data_t;
      
      typedef ColeVishkinMessage<OsModel, Radio> message_t;
      
      int init(typename Radio::self_pointer_t radio,
            typename Neighborhood::self_pointer_t neighborhood,
            typename Debug::self_pointer_t debug,
            typename LabelDebug::self_pointer_t labeldebug) {
         radio_ = radio;
         neighborhood_ = neighborhood;
         debug_ = debug;
         labeldebug_ = labeldebug;
         return init();
      }
      
      int init() {
         radio_->template reg_recv_callback<self_type, &self_type::on_receive>(this);
         on_start();
         return OsModel::SUCCESS;
      }
      
      int destruct() {
         return OsModel::SUCCESS;
      }
      
      color_t color() { return color_; }
      
      // ------------------------
      
      void on_start() {
         set_color( radio_->id() );
         
         // Iterate over all tree childs, *iter is the child's address
         for(typename Neighborhood::Neighbors::iterator iter = neighborhood_->topology().begin();
               iter != neighborhood_->topology().end(); ++iter) {
            
            // Send our current color to child
            message_t msg;
            msg.set_command_type(message_t::CMD_COLOR);
            msg.set_color( color() );
            radio_->send(*iter, msg.size(), msg.data());
         }
         
      }
      
      void on_receive(node_id_t from, size_t len, block_data_t* data) {
         message_t *msg = reinterpret_cast<message_t*>(data);
         
         if(msg->command_type() == message_t::CMD_COLOR) {
            debug_->debug("%d received color '%d' from node %d.\n", radio_->id(), msg->color(), from);
         }
      }
      
      // ------------------------
      
      
   private:
      void set_color(color_t c) {
         color_ = c;
         labeldebug_->debug("%d", color_);
      }
      
      color_t color_;
      typename Radio::self_pointer_t radio_;
      typename Neighborhood::self_pointer_t neighborhood_;
      typename Debug::self_pointer_t debug_;
      typename LabelDebug::self_pointer_t labeldebug_;
};
      

#endif // COLE_VISHKIN_H

/* vim: set ts=3 sw=3 tw=78 expandtab :*/
